/* play.c - play a sound file on the speaker
 **
 ** Copyright (C) 1989 by Jef Poskanzer.
 **
 ** Permission to use, copy, modify, and distribute this software and its
 ** documentation for any purpose and without fee is hereby granted, provided
 ** that the above copyright notice appear in all copies and that both that
 ** copyright notice and this permission notice appear in supporting
 ** documentation.  This software is provided "as is" without express or
 ** implied warranty.
 */
#ifdef sparc
#include <stdio.h>
#include <fcntl.h>
#include <sys/file.h>
#include <sys/signal.h>
#include "libsst.h"

#define MYBUF 256

int sst_fd;

void (*sighup_handler)();
void (*sigint_handler)();

play_sound(sound_file, volume)
    char *sound_file;
    int volume;
{
    int rrtn, wrtn;
    unsigned char buf[MYBUF];
    int file_fd;
    int sighandler();
    
    sst_fd = sst_open(volume,0);
    sighup_handler = signal( SIGHUP, sighandler );
    sigint_handler = signal( SIGINT, sighandler );
    
    file_fd = open( sound_file, O_RDONLY );
    if ( file_fd < 0 )
    {
	perror( sound_file );
	return( 1 );
    }
#ifndef SUNOS4_0_3
    {
    int err;
    Audio_hdr Dev_hdr;
    Audio_hdr File_hdr;

    err = audio_get_play_config( sst_fd, &Dev_hdr );

    if ( err != AUDIO_SUCCESS )
    {
	perror( "Not a valid audio device" );
	return( 1 );
    }

    err = audio_read_filehdr( file_fd, &File_hdr, (char *) NULL, 0 );

    if ( err != AUDIO_SUCCESS )
    {
	perror( "Not a valid audio file" );
	return( 1 );
    }

    /* Shameless pilfered from /usr/demo/SOUND/play.c */
    if ( audio_cmp_hdr(&Dev_hdr, &File_hdr) != 0 ) {
	if ( audio_drain( sst_fd, FALSE) != AUDIO_SUCCESS) {
	    perror( "draining audio device" );
	    return( 1 );
	}
    /* Fancy code to reconfigure audio device for given file header omitted */
    }
    }
#endif
    
    for ( ; ; )
    {
	rrtn = read( file_fd, buf, MYBUF );
	if ( rrtn < 0 )
	{
	    perror( "read" );
	    return( 1 );
	}
	if ( rrtn == 0 )
	    break;
	
	for ( ; ; )
	{
	    wrtn = write( sst_fd, buf, rrtn );
	    if ( wrtn < 0 )
	    {
		perror( "write" );
		return( 1 );
	    }
	    if ( wrtn != 0 )
		break;
	    usleep( 1000 );
	}
	if ( wrtn != rrtn )
	{
	    fprintf( stderr, "play: rrtn = %d, wrtn = %d\n", rrtn, wrtn );
	    return( 1 );
	}
    }
    
    close(file_fd);
#ifndef SUNOS4_0_3
    if ( audio_drain( sst_fd, FALSE) != AUDIO_SUCCESS) {
	perror( "draining audio device" );
	return( 1 );
    }
#endif
    sst_close( sst_fd );
    (void) signal( SIGHUP, sighup_handler );
    (void) signal( SIGINT, sigint_handler );
}

static int
sighandler(sig, code, scp, addr)
    int sig, code;
    struct sigcontext *scp;
    char *addr;
{
    sst_close( sst_fd );
    if (sig == SIGHUP && sighup_handler)
	(*sighup_handler)(sig, code, scp, addr);
    else if (sigint_handler)
	(*sigint_handler)(sig, code, scp, addr);
    exit( 1 );
}
#else
play_sound(sound_file, volume)
    char *sound_file;
    int volume;
{
    return( 1 );
}
#endif
